<?php

namespace EventPortal\Providers;

use EventPortal\Domain\Event\Models\Event;
use EventPortal\Exceptions\Handler;
use Illuminate\Contracts\Debug\ExceptionHandler;
use Illuminate\Support\Facades\Route;
use Illuminate\Support\ServiceProvider;
use Inertia\Inertia;

class AppServiceProvider extends ServiceProvider
{
    private const CONFIGS = [
        'defaults.event.fields' => '/defaults/event/fields.php',
        'defaults.event.tableColumns' => '/defaults/event/tableColumns.php',
        'defaults.user.fields' => '/defaults/user/fields.php',
        'defaults.user.tableColumns' => '/defaults/user/tableColumns.php',
        'defaults.delegate.fields' => '/defaults/delegate/fields.php',
        'defaults.delegate.tableColumns' => '/defaults/delegate/tableColumns.php',
    ];

    /** @var string */
    protected $root = __DIR__.'/../..';

    public function boot(): void
    {
        $this->app->booted(function (): void {
            $this->loadRoutesFrom("{$this->root}/routes/routes.php");
        });

        $this->loadViewsFrom("{$this->root}/resources/views", 'EventPortal');

        $this->loadMigrationsFrom("{$this->root}/database/migrations");

        $this->loadFactoriesFrom("{$this->root}/database/factories");

        $this->app->bind(ExceptionHandler::class, Handler::class);

        $this->publishes([
            "{$this->root}/tables/delegate.php" => '/tables/delegates.php',
            "{$this->root}/tables/event.php" => '/tables/event.php',
            "{$this->root}/tables/event-delegate.php" => '/tables/event-delegate.php',
        ], 'tables');

        $this->publishes([
            "{$this->root}/forms/delegate.php" => '/forms/delegates.php',
            "{$this->root}/forms/event.php" => '/forms/event.php',
            "{$this->root}/forms/event-delegate.php" => '/forms/event-delegate.php',
        ], 'forms');

        $this->publishes([
            "{$this->root}/config/permission.php" => config_path('/permission.php'),
            "{$this->root}/config/state-machine.php" => config_path('/state-machine.php'),
        ], 'permissions');

        $this->app->bind(\Illuminate\Contracts\Debug\ExceptionHandler::class, \EventPortal\Exceptions\Handler::class);

        Inertia::setRootView('EventPortal::app');

        Inertia::share([
            'errors' => function () {
                return session('errors') ? session('errors')->getBag('default')->getMessages() : (object) [];
            },
        ]);

        Route::model('event', Event::class);
    }

    public function register(): void
    {
    }

    /**
     * @param string $path
     * @param string $key
     * @phpcsSuppress SlevomatCodingStandard.TypeHints.ParameterTypeHint.MissingNativeTypeHint
     */
    protected function mergeConfigFrom($path, $key): void
    {
        $config = $this->app['config']->get($key, []);

        $this->app['config']->set($key, $this->mergeConfig(require $path, $config));
    }

    /**
     * Allows configs that are nested with or without keys to be merged
     */
    private function mergeConfig(array $original, array $merging): array
    {
        $array = array_replace($original, $merging);

        foreach ($original as $key => $value) {
            if (!is_array($value)) {
                continue;
            }

            if (!Arr::exists($merging, $key)) {
                continue;
            }

            if (is_numeric($key)) {
                continue;
            }

            $array[$key] = $this->mergeConfig($value, $merging[$key]);
        }

        return $array;
    }
}
